import json

from upvest.config import API_VERSION, UPVEST_API_TARGET
from upvest.model import Assets, Wallets, Transactions
from upvest.authentication import OAuth

class UpvestClienteleAPI(object):
    def __init__(self, client_id, client_secret, username, password, base_url=None):
        base_url = base_url or UPVEST_API_TARGET
        self.auth_instance = OAuth(client_id=client_id, client_secret=client_secret, username=username, password=password, base_url=base_url)
        self.assets = Assets(self.auth_instance)
        self.wallets = Wallets(self.auth_instance)



import time
import json
import hmac
import hashlib
import requests

from upvest.config import ENCODING
from upvest.config import OAUTH_PATH
from upvest.config import API_VERSION
from upvest.config import GRANT_TYPE
from upvest.config import SCOPE

class KeyAuth(object):
    def __init__(self, api_key=None, api_secret=None, api_passphrase=None, base_url=None, **kwargs):
        self.api_key = api_key
        self.api_secret = api_secret
        self.api_passphrase = api_passphrase
        self.base_url = base_url

    def get_headers(self, method=None, path=None, body=None, **kwargs):
        body = json.dumps(body) if body else None
        timestamp = str(int(time.time()))
        # Compose the message as a concatenation of all info we are sending along with the request
        message = timestamp + method + API_VERSION + path + (body or '')
        # Generate signature, in order to prevent manipulation of payload in flight
        signature = hmac.new(str.encode(self.api_secret),
                             msg=message.encode(ENCODING),
                             digestmod=hashlib.sha512).hexdigest()
        # Generate message headers
        headers = {
            'Content-Type': 'application/json',
            'X-UP-API-Key': self.api_key,
            'X-UP-API-Signature': signature,
            'X-UP-API-Timestamp': timestamp,
            'X-UP-API-Passphrase': self.api_passphrase,
            'X-UP-API-Signed-Path': API_VERSION + path,
        }
        return headers

class OAuth(object):
    def __init__(self, client_id=None, client_secret=None, username=None, password=None, base_url=None, **kwargs):
        self.path = API_VERSION + OAUTH_PATH
        self.client_id = client_id
        self.client_secret = client_secret
        self.username = username
        self.password = password
        self.base_url = base_url

    def get_headers(self, **req_params):
        # Set header content-type to x-www-form-urlencoded
        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
        }
        body = {
            'grant_type': GRANT_TYPE,
            'scope': SCOPE,
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'username': self.username,
            'password': self.password,
        }
        # send x-www-form-urlencoded payload to clientele API.
        r = requests.post(self.base_url + self.path, data=body, headers=headers)
        # Retrieve and return OAuth token
        oauth_token = r.json()['access_token']
        headers = {
            'Authorization': f'Bearer {oauth_token}',
            'Content-Type': 'application/json',
        }
        return headers



from upvest.config import API_VERSION, UPVEST_API_TARGET
from upvest.authentication import KeyAuth
from upvest.model import UserInstance, Users, Assets

class UpvestTenancyAPI(object):
    def __init__(self, api_key, api_secret, api_passphrase, base_url=None):
        base_url = base_url or UPVEST_API_TARGET
        self.auth_instance = KeyAuth(api_key=api_key, api_secret=api_secret, api_passphrase=api_passphrase, base_url=base_url)
        self.users = Users(self.auth_instance)
        self.assets = Assets(self.auth_instance)



import json
import requests

from upvest.config import API_VERSION
from upvest.exceptions import InvalidRequest

class Response(object):
    def __init__(self, result):
        self.status_code = result.status_code
        self.raw = result
        try:
            self.json = result.json()
            try:
                self.data = self.json['results']
            except:
                self.data = self.json
        except:
            #raise ValueError
            self.data = None

# Request object
class Request(object):
    def __init__(self):
        pass

    def _request(self, **req_params):
        # Set request parameters
        body = req_params.get('body', None)
        path = req_params.get('path')
        method = req_params.get('method')
        if body is not None:
            for value in body.values():
                if isinstance(value, int):
                    pass
                else:
                    try:
                        value.encode('ascii')
                    except UnicodeEncodeError:
                        raise Exception('Forbidden characters present, please remove')
        # Instantiate the respectively needed auth instance
        auth_instance = req_params.get('auth_instance')
        authenticated_headers = auth_instance.get_headers(**req_params)
        # Execute request with authenticated headers
        request_url = auth_instance.base_url + API_VERSION + path
        response = requests.request(method, request_url, json=body, headers=authenticated_headers)
        if response.status_code >= 300:
            raise InvalidRequest(response.text)
        else:
            return response

    def post(self, **req_params):
        req_params['method'] = 'POST'
        return self._request(**req_params)
    
    def get(self, **req_params):
        req_params['method'] = 'GET'
        return self._request(**req_params)
    
    def patch(self, **req_params):
        req_params['method'] = 'PATCH'
        return self._request(**req_params)
    
    def delete(self, **req_params):
        req_params['method'] = 'DELETE'
        return self._request(**req_params)



from base64 import b64encode
import hashlib

from urllib.parse import urlsplit, urlunsplit, urlencode, parse_qs
from upvest.utils import Request, Response
from upvest.config import API_VERSION

# Endpoint related objects
class UserInstance(object):
    def __init__(self, auth_instance, username, recovery_kit=None):
        self.path = '/tenancy/users/'
        self.auth_instance = auth_instance
        self.username = username
        self.recovery_kit = recovery_kit

    def update(self, password, new_password):
        # Provide current and new password
        body = {
            'old_password': password,
            'new_password': new_password,
        }
        response = Response(Request().patch(auth_instance=self.auth_instance, path=self.path + self.username, body=body))
        username = response.data['username']
        return UserInstance(response, username)

    def delete(self):
        # Deregister a user
        return None

class Users(object):
    def __init__(self, auth_instance):
        self.path = '/tenancy/users/'
        self.auth_instance = auth_instance

    def create(self, username, password):
        # Set username and password for the user
        body = {
            'username': username,
            'password': password,
        }
        response = Response(Request().post(auth_instance=self.auth_instance, path=self.path, body=body))
        username = response.data['username']
        recovery_kit = response.data['recoverykit']
        return UserInstance(self.auth_instance, username, recovery_kit)

    def get(self, username):
        response = Response(Request().get(auth_instance=self.auth_instance, path=self.path + username))
        username = response.data['username']
        return UserInstance(self.auth_instance, username)

    def list(self, count):
        # Retrieve subset of users
        MAX_PAGE_SIZE = 100
        array_of_users = []
        path = f'/tenancy/users/?page_size={MAX_PAGE_SIZE}'
        listed_count = 0

        while listed_count < count:
            response = Response(Request().get(auth_instance=self.auth_instance, path=path))
            for user in response.data:
                username = user['username']
                array_of_users.append(UserInstance(self.auth_instance, username))
                listed_count += 1
                if listed_count >= count:
                    break
            if response.raw.json()['next']:
                page_size = min(MAX_PAGE_SIZE, count - listed_count)
                path = response.raw.json()['next'].split(API_VERSION)[-1]
                path_parts = list(urlsplit(path))
                query = parse_qs(path_parts[3])
                query['page_size'] = [page_size]
                path_parts[3] = urlencode(query, doseq=True)
                path = urlunsplit(path_parts)
            else:
                break
        return array_of_users

    def all(self):
        MAX_PAGE_SIZE = 100
        array_of_users = []
        path = '/tenancy/users/?{MAX_PAGE_SIZE}'

        while True:
            response = Response(Request().get(auth_instance=self.auth_instance, path=path))
            for user in response.data:
                username = user['username']
                array_of_users.append(UserInstance(self.auth_instance, username))
            if response.raw.json()['next']:
                path = response.raw.json()['next'].split(API_VERSION)[-1]
                path_parts = list(urlsplit(path))
                query = parse_qs(path_parts[3])
                query['page_size'] = [MAX_PAGE_SIZE]
                path_parts[3] = urlencode(query, doseq=True)
                path = urlunsplit(path_parts)
            else:
                break
        return array_of_users

class AssetInstance(object):
    def __init__(self,**asset_attr):
        self.id = asset_attr['id']
        self.name = asset_attr['name']
        self.symbol = asset_attr['symbol']
        self.exponent = asset_attr['exponent']
        self.protocol = asset_attr['protocol']
        self.metadata = asset_attr['metadata']

class Assets(object):
    def __init__(self, auth_instance):
        self.path = '/assets/'
        self.auth_instance = auth_instance

    def all(self):
        MAX_PAGE_SIZE = 100
        array_of_assets = []
        path = '/assets/?{MAX_PAGE_SIZE}'

        while True:
            response = Response(Request().get(auth_instance=self.auth_instance, path=path))
            for asset in response.data:
                array_of_assets.append(AssetInstance(**asset))
            if response.raw.json()['next']:
                path = response.raw.json()['next'].split(API_VERSION)[-1]
                path_parts = list(urlsplit(path))
                query = parse_qs(path_parts[3])
                query['page_size'] = [MAX_PAGE_SIZE]
                path_parts[3] = urlencode(query, doseq=True)
                path = urlunsplit(path_parts)
            else:
                break
        return array_of_assets

class ECDSASignature:
    def __init__(self, signed_hash, j):
        self.algorithm = j["algorithm"]
        if self.algorithm != "ECDSA":
            raise ValueError(f"unsupported algorithm: {self.algorithm}")

        self.curve = j["curve"]
        self.x = j["public_key"]["x"]
        self.y = j["public_key"]["y"]
        self.r = j["r"]
        self.s = j["s"]
        self.signed_hash = signed_hash

        if self.curve == "secp256k1":
            self.recover = j["recover"]
        else:
            # TODO: is pubkey recovery possible for other curves?
            pass

class WalletInstance(object):
    def __init__(self, auth_instance, **wallet_attr):
        self.auth_instance = auth_instance
        self.transactions = Transactions(auth_instance, wallet_attr['id'])
        self.id = wallet_attr['id']
        self.path = f'/kms/wallets/{self.id}'
        self.balances = wallet_attr['balances']
        self.protocol  = wallet_attr['protocol']
        self.address = wallet_attr['address']
        self.status = wallet_attr['status']

    def sign(self, password, message=None, to_sign_hash=None, hash_algorithm='SHA256'):
        if to_sign_hash is None and message is not None:
            if not isinstance(message, (bytes, bytearray)):
                raise TypeError("message argument is not a bytes-like object")

            if hash_algorithm is 'SHA256':
                to_sign_hash = hashlib.sha256(message).digest()
            else:
                raise ValueError(f'unsupported hash_algorithm: {hash_algorithm}')

        if to_sign_hash is None:
            raise ValueError(f'neither message nor to_sign_hash were provided')

        body = {
            'password': password,
            'to_sign': str(b64encode(to_sign_hash), "UTF-8"),
            'output_format': 'int',
        }
        rsp = Response(Request().post(auth_instance=self.auth_instance, path=f'{self.path}/sign', body=body))
        return ECDSASignature(signed_hash=to_sign_hash, j=rsp.data)

class Wallets(object):
    def __init__(self, auth_instance):
        self.path = '/kms/wallets/'
        self.auth_instance = auth_instance

    def create(self, asset_id, password):
        # Get desired asset id from assets list
        # Provide password and asset_id for wallet creation
        body = {
            'password': password,
            'asset_id': asset_id,
        }
        response = Response(Request().post(auth_instance=self.auth_instance, path=self.path, body=body))
        return WalletInstance(self.auth_instance, **response.data)

    def get(self, wallet_id):
        # Retrieve specific wallet for a user
        response = Response(Request().get(auth_instance=self.auth_instance, path=self.path + wallet_id))
        return WalletInstance(self.auth_instance, **response.data)

    def list(self, count):
        # Retrieve subset of wallets
        MAX_PAGE_SIZE = 100
        array_of_wallets = []
        path = f'/kms/wallets/?page_size={MAX_PAGE_SIZE}'
        listed_count = 0

        while listed_count < count:
            response = Response(Request().get(auth_instance=self.auth_instance, path=path))
            for wallet in response.data:
                array_of_wallets.append(WalletInstance(self.auth_instance, **wallet))
                listed_count += 1
                if listed_count >= count:
                    break
            if response.raw.json()['next']:
                page_size = min(MAX_PAGE_SIZE, count - listed_count)
                path = response.raw.json()['next'].split(API_VERSION)[-1]
                path_parts = list(urlsplit(path))
                query = parse_qs(path_parts[3])
                query['page_size'] = [page_size]
                path_parts[3] = urlencode(query, doseq=True)
                path = urlunsplit(path_parts)
            else:
                break
        return array_of_wallets

    def all(self):
        # Retrieve subset of wallets
        MAX_PAGE_SIZE = 100
        array_of_wallets = []
        path = f'/kms/wallets/?page_size={MAX_PAGE_SIZE}'

        while True:
            response = Response(Request().get(auth_instance=self.auth_instance, path=path))
            for wallet in response.data:
                array_of_wallets.append(WalletInstance(self.auth_instance, **wallet))
            if response.raw.json()['next']:
                path = response.raw.json()['next'].split(API_VERSION)[-1]
                path_parts = list(urlsplit(path))
                query = parse_qs(path_parts[3])
                query['page_size'] = [MAX_PAGE_SIZE]
                path_parts[3] = urlencode(query, doseq=True)
                path = urlunsplit(path_parts)
            else:
                break
        return array_of_wallets

class TransactionInstance(object):
    def __init__(self, **transaction_attr):
        self.path = '/kms/wallets/'
        self.id = transaction_attr['id']
        self.txhash = transaction_attr['txhash']
        self.sender = transaction_attr['sender']
        self.recipient = transaction_attr['recipient']
        self.quantity = int(transaction_attr['quantity'])
        self.fee = int(transaction_attr['fee'])
        self.status = transaction_attr['status']

class Transactions(object):
    def __init__(self, auth_instance, wallet_id):
        self.path = '/kms/wallets/'
        self.auth_instance = auth_instance
        self.wallet_id = wallet_id

    def create(self, password, asset_id, quantity, fee, recipient):
        # Provide password and asset_id for wallet creation
        body = {
            'password': password,
            'asset_id': asset_id,
            'quantity': str(quantity),
            'fee': str(fee),
            'recipient': recipient,
        }
        response = Response(Request().post(auth_instance=self.auth_instance, path=f'{self.path}{self.wallet_id}/transactions/', body=body))
        return TransactionInstance(**response.data)

    def get(self, id):
        # Define tx endpoint
        response = Response(Request().get(auth_instance=self.auth_instance, path=f'{self.path}{self.wallet_id}/transactions/{id}'))
        return TransactionInstance(**response.data)

    def list(self, count):
        # Retrieve subset of wallets
        MAX_PAGE_SIZE = 100
        array_of_transactions = []
        path = f'{self.path}{self.wallet_id}/transactions/?page_size={MAX_PAGE_SIZE}'
        listed_count = 0

        while listed_count < count:
            response = Response(Request().get(auth_instance=self.auth_instance, path=path))
            for transaction in response.data:
                array_of_transactions.append(TransactionInstance(**transaction))
                listed_count += 1
                if listed_count >= count:
                    break
            if response.raw.json()['next']:
                page_size = min(MAX_PAGE_SIZE, count - listed_count)
                path = response.raw.json()['next'].split(API_VERSION)[-1]
                path_parts = list(urlsplit(path))
                query = parse_qs(path_parts[3])
                query['page_size'] = [page_size]
                path_parts[3] = urlencode(query, doseq=True)
                path = urlunsplit(path_parts)
            else:
                break
        return array_of_transactions

    def all(self):
        MAX_PAGE_SIZE = 100
        array_of_transactions = []
        path = f'{self.path}{self.wallet_id}/transactions/?page_size={MAX_PAGE_SIZE}'

        while True:
            response = Response(Request().get(auth_instance=self.auth_instance, path=path))
            for transaction in response.data:
                array_of_transactions.append(TransactionInstance(**transaction))
            if response.raw.json()['next']:
                path = response.raw.json()['next'].split(API_VERSION)[-1]
                path_parts = list(urlsplit(path))
                query = parse_qs(path_parts[3])
                query['page_size'] = [MAX_PAGE_SIZE]
                path_parts[3] = urlencode(query, doseq=True)
                path = urlunsplit(path_parts)
            else:
                break
        return array_of_transactions



import os

API_VERSION = '1.0'
OAUTH_PATH = '/clientele/oauth2/token'
ENCODING = 'utf-8'
GRANT_TYPE = 'password'
SCOPE = 'read write echo transaction'

UPVEST_API_TARGET = os.getenv('UPVEST_API_TARGET', default = 'https://api.playground.upvest.co/')



import os
import setuptools

_README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

_REQUIREMENTS = [
    line.strip() for line in open(os.path.join(os.path.dirname(__file__),'requirements.upvest.txt')).readlines()
]

_CLASSIFIERS = [
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 3.7',
    'Topic :: Software Development :: Libraries :: Python Modules',
]

_KEYWORDS = [
    'api', 
    'upvest', 
    'bitcoin', 
    'ethereum', 
    'oauth2', 
    'client',
]

_PACKAGES = setuptools.find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"])

setuptools.setup(
    name="upvest",
    version="0.0.5",
    author="Upvest GmbH",
    author_email="tech@upvest.co",
    maintainer="Alexander Reichhardt",
    maintainer_email="alexander@upvest.co",
    description="Upvest API client library",
    keywords=_KEYWORDS,
    license='MIT',
    long_description=_README,
    long_description_content_type="text/markdown",
    url="https://github.com/toknapp/python-sdk-upvest/",
    packages=_PACKAGES,
    include_package_data=True,
    classifiers=_CLASSIFIERS,
    install_requires=_REQUIREMENTS,
)
